/*
 * XMLNewsTag.java
 *
 * Created on March 15, 2002, 3:56 PM
 */

import java.io.*;
import java.util.*;
import javax.servlet.jsp.*;
import javax.servlet.jsp.tagext.*;

/**
 * Custom tag which is used to print xml news articles encoded in wml.  There
 * are 4 required attributes for this tag to work.  maxArticles is the maximum
 * number of articles that is to be output by the tag.  baseUrl is the url 
 * to the base of the document.  newsServer is the url to the news server that 
 * has the xml encoded news listings.  country is the country name that is to
 * be queried for against the news server.
 *
 * @author  andrewm
 * @version 1.0
 */
public class XMLNewsTag extends BodyTagSupport
{
    // Member variables which represent the attributes required by this tag
    private int maxArticles;
    private String baseUrl;
    private String newsServer;
    private String country;
    
    /**
     * getMaxArticles
     *
     * @return int The current value of maxArticles
     */
    public int getMaxArticles()
    {
	return maxArticles;
    }
    
    /**
     * getBaseUrl
     *
     * @return String The current value of baseUrl
     */
    public String getBaseUrl()
    {
	return baseUrl;
    }
    
    /**
     * getNewsServer
     *
     * @return String The current value of newsServer
     */
    public String getNewsServer()
    {
	return newsServer;
    }
    
    /**
     * getCountry
     *
     * @return String The current value of country
     */
    public String getCountry()
    {
	return country;
    }
    
    /**
     * setMaxArticles
     *
     * @param _maxArticles The new value for maxArticles
     */
    public void setMaxArticles( int _maxArticles )
    {
	maxArticles = _maxArticles;
    }
    
    /**
     * setBaseUrl
     *
     * @param _baseUrl The new value for baseUrl
     */
    public void setBaseUrl( String _baseUrl )
    {
	baseUrl = _baseUrl;
    }
    
    /**
     * setNewsServer
     *
     * @param _newsServer The new value for newsServer
     */
    public void setNewsServer( String _newsServer )
    {
	newsServer = _newsServer;
    }
    
    /**
     * setCountry
     *
     * @param _country The new value for country
     */
    public void setCountry( String _country )
    {
	country = _country;
    }
    
    /**
     * Called when the start tag of an XMLNewsTag is encountered.
     *
     * @return int Code which represents the action to be taken after this method
     * @exception JspException if a JSP exception has occurred
     */
    public int doStartTag() throws JspException
    {
	try
	{
	    // Retrieve the JspWriter
	    JspWriter out = pageContext.getOut();
	    
	    // Create the content retriever
	    ContentRetriever retriever = ContentRetriever.getInstance();
	    
	    // Retrieve the content from the news server for the specified country
	    InputStream xmlStream = retriever.retrieveContent( newsServer + country );
	    
	    // Check to see if the xml was successfully retrieved
	    if ( xmlStream != null )
	    {
		// if the xml stream isn't null then parse it
		
		// Create a parser instance with the xml stream
		XMLNewsParser parser = new XMLDOMNewsParser( xmlStream );
		
		// parse the stream
		if ( parser.parse() )
		{
		    // check to see if there were any articles retrieved
		    if ( parser.getNumArticles() > 0 )
		    {
			int articleNum = 0;
			
			// Retrieve up to the maximum number of articles
			Enumeration enums = parser.getXMLNewsArticles( maxArticles );
			XMLNewsArticle article;
			
			// Loop through the articles that were returned
			while( enums.hasMoreElements() )
			{
			    // count the number of articles
			    articleNum++;
			    
			    // extract the next article from the enumeration
			    article = (XMLNewsArticle)enums.nextElement();
			    
			    // encode characters that may cause problems
			    article.encode();
			    
			    // Print the wml for the articles
			    out.println( "<card id=\"article" + articleNum + "\">" );
			    out.println( "<p>" + article.getHarvesttime() + ":</p>" );
			    out.println( "<p>" + article.getHeadline() );
			    
			    // if there are remaining articles
			    if ( enums.hasMoreElements() )
			    {
				out.println( "<do type=\"accept\" label=\"next\">" );
				out.println( "	 <go href=\"#article" + ( articleNum + 1 ) + "\"/>" );
				out.println( "</do>" );
			    }
			    // if this is the last article then add the link to home
			    else
			    {
				out.println( "<do type=\"accept\" label=\"home\">" );
				out.println( "	 <go href=\"" + baseUrl + "\"/>" );
				out.println( "</do>" );
			    }
			    out.println( "</p>" );
			    out.println( "</card>" );
			}
		    }
		    // No news was returned from the server
		    else
		    {
			out.println( "<card id=\"no_news\" title=\"No News\">" );
			out.println( "<p>Sorry, no news for the country " + country + "." );
			out.println( "<do type=\"accept\" label=\"OK\">" );
			out.println( "   <go href=\"" + baseUrl + "\"/>" );
			out.println( "</do>" );
			out.println( "</p>" );
			out.println( "</card>" );
		    }
		}
		// There was an error parsing the xml stream
		else
		{
		    out.println( "<card id=\"error_page\" title=\"Error\">" );
		    out.println( "<p>Error parsing news for " +  country + ".</p>" );
		    out.println( "</card>" );
		}
	    }
	    // There was an error retrieving the xml stream
	    else
	    {
		out.println( "<card id=\"error_page\" title=\"Error\">" );
		out.println( "<p>Error retrieving news for " + country + ".</p>" );
		out.println( "</card>" );
	    }
	}
	catch( Throwable t )
	{
	    t.printStackTrace();
	}
	finally
	{
	    return SKIP_BODY;
	}
    }
    
    /**
     * Called when the end tag of an XMLNewsTag is encountered.
     *
     * @return int Code which represents the action to be taken after this method
     * @exception JspException if a JSP exception has occurred
     */
    public int doEndTag()
    {
	return EVAL_PAGE;
    }
}
